/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of CG IDLE demo for the TOSHIBA
 *          'TMPM037' Device Series 
 * @version V0.100
 * @date    2014/06/28
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#include "main.h"
#include "tmpm037_int.h"

/**
  * @brief  The main function of demo
  * @param  None
  * @retval None
  */
    
#define EN_TBEN_SET           ((uint32_t)0x00000080)
#define EN_TBEN_CLEAR         ((uint32_t)0xFFFFFF7F)

#define CR_I2TB_SET           ((uint32_t)0x00000008)
#define CR_I2TB_CLEAR         ((uint32_t)0xFFFFFFF7)
#define CR_TBSYNC_SET         ((uint32_t)0x00000020)
#define CR_TBSYNC_CLEAR       ((uint32_t)0xFFFFFFDF)
#define CR_TBWBF_SET          ((uint32_t)0x00000080)
#define CR_TBWBF_CLEAR        ((uint32_t)0xFFFFFF7F)
#define CR_CSSEL_SET          ((uint32_t)0x00000001)
#define CR_CSSEL_CLEAR        ((uint32_t)0xFFFFFFFE)
#define CR_TRGSEL_CLEAR       ((uint32_t)0xFFFFFFFD)
#define CR_BIT6_CLEAR         ((uint32_t)0xFFFFFFBF)

#define MOD_BIT7_CLEAR        ((uint32_t)0xFFFFFF7F)
#define MOD_CLK_CLE_CLEAR     ((uint32_t)0xFFFFFFF0)
#define MOD_TBCPM_CLEAR       ((uint32_t)0xFFFFFF8F)
#define MOD_TBCP_SET          ((uint32_t)0x00000040)
#define MOD_TBCP_CLEAR        ((uint32_t)0xFFFFFFBF)

#define TB_ST_MASK            ((uint32_t)0x00000007)
#define TB_IM_MASK            ((uint32_t)0x00000007)
 
    
int main(void)
{
    TMRB_InitTypeDef m_tmrb;
    
    SW_Init();
    LED_Init();
    hardware_init(UART_RETARGET);
    
    m_tmrb.Mode = TMRB_INTERVAL_TIMER;
    m_tmrb.ClkDiv = TMRB_CLK_DIV_512;
    m_tmrb.TrailingTiming = TMRB_1MS;   /* periodic time is 1ms */
    m_tmrb.UpCntCtrl = TMRB_AUTO_CLEAR;
    m_tmrb.LeadingTiming = TMRB_1MS;    /* periodic time is 1ms */
    
    TMRB_Enable(TSB_TB0);
    TMRB_Init(TSB_TB0, &m_tmrb);
    TMRB_SetINTMask(TSB_TB0, TMRB_MASK_MATCH_LEADINGTIMING_INT);        /*leadingtiming is not used in TMRB_INTERVAL_TIMER mode */
    TMRB_SetIdleMode(TSB_TB0, ENABLE);
    
    NVIC_ClearPendingIRQ(ExtINTSrc_IRQn);
    NVIC_EnableIRQ(INTTMRB0_IRQn);
   	
    while (1U) {
        if (SW_Get(SW1) == 1U) {
            LED_On(LED_ALL);
            common_uart_disp("IDLE MODE\r\n");
            Delay(); /* Delay sometime to let UART print string. */
            TMRB_SetRunState(TSB_TB0, TMRB_STOP);
            TMRB_SetRunState(TSB_TB0, TMRB_RUN);
            enter_IDLE();
            LED_Off(LED_ALL);
            common_uart_disp("NORMAL MODE\r\n");            
            
      } else {
            LED_Off(LED_ALL);
            common_uart_disp("NORMAL MODE\r\n");
        }
      }
}

/**
  * @brief  Enter the IDLE low power mode
  * @param  None
  * @retval None
  */
void enter_IDLE(void)
{
    /* Set standby mode as IDLE */
    CG_SetSTBYMode(CG_STBY_MODE_IDLE);
    __DSB();
    __WFI();
}

/**
  * @brief  Delay sometime.
  * @param  None
  * @retval None
  */
void Delay(void)
{
    volatile uint32_t i = 0x0U; 
    for(i = 0xFFFU; i>0; i--);
}

void TMRB_Enable(TSB_TB_TypeDef * TBx)
{
    /* Check the parameters */
    assert_param(IS_TMRB_ALL_PERIPH(TBx));
    /* Set TBxEN<TBEN> to enable TBx */
    TBx->EN |= EN_TBEN_SET;
}

void TMRB_Init(TSB_TB_TypeDef * TBx, TMRB_InitTypeDef * InitStruct)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_TMRB_ALL_PERIPH(TBx));
    assert_param(IS_TMRB_MODE(InitStruct->Mode));
    if (InitStruct->Mode != 0U) {
        assert_param(IS_TMRB_CLK_DIV(InitStruct->ClkDiv));
    } else {
        /* Do nothing */
    }
    assert_param(IS_TMRB_VALUE(InitStruct->TrailingTiming));
    assert_param(IS_TMRB_UC_CTRL(InitStruct->UpCntCtrl));
    assert_param(IS_TMRB_VALUE(InitStruct->LeadingTiming));
    assert_param(IS_VALID_LEADINGTIMING(InitStruct->LeadingTiming, InitStruct->TrailingTiming));

    /* Configure source clock for TBx */
    tmp = TBx->MOD;
    tmp &= MOD_BIT7_CLEAR;
    tmp &= MOD_CLK_CLE_CLEAR;
    if (InitStruct->Mode != 0U) {
        /* Use internal clock, set the prescaler */
        tmp |= InitStruct->ClkDiv;
    } else {
        /* Use external clock */
        tmp |= InitStruct->Mode;
    }
    /* Set up-counter running mode */
    tmp |= InitStruct->UpCntCtrl;
    tmp |= MOD_TBCP_SET;
    TBx->MOD = tmp;

    /* Write leadingtiming into RG0 */
    TBx->RG0 = InitStruct->LeadingTiming;

    /* Write trailingtiminginto RG1 */
    TBx->RG1 = InitStruct->TrailingTiming;
}

void TMRB_SetINTMask(TSB_TB_TypeDef * TBx, uint32_t INTMask)
{
    /* Check the parameters */
    assert_param(IS_TMRB_ALL_PERIPH(TBx));
    assert_param(IS_TMRB_INT_MASK(INTMask));

    /* Mask the specified interrupt */
    TBx->IM = INTMask;
}

void TMRB_SetRunState(TSB_TB_TypeDef * TBx, uint32_t Cmd)
{
    /* Check the parameters */
    assert_param(IS_TMRB_ALL_PERIPH(TBx));
    assert_param(IS_TMRB_CMD(Cmd));

    /* Write command into RUN register */
    TBx->RUN = Cmd;
}

void TMRB_SetIdleMode(TSB_TB_TypeDef * TBx, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRB_ALL_PERIPH(TBx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    tmp = TBx->CR;
    tmp &= CR_BIT6_CLEAR;
    if (NewState == ENABLE) {
        /* Set TBxCR<I2TB> to enable TBx in system idle mode */
        TBx->CR = tmp | CR_I2TB_SET;
    } else {
        /* Clear TBxCR<I2TB> to disable TBx in system idle mode */
        TBx->CR = tmp & CR_I2TB_CLEAR;
    }
}


#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif
